"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Message = void 0;
const zhin_1 = require("zhin");
var Message;
(function (Message) {
    function parseSegmentsFromCqCode(template) {
        const result = [];
        const reg = /(\[CQ:[!\]]+])/;
        while (template.length) {
            const [match] = template.match(reg) || [];
            if (!match)
                break;
            const index = template.indexOf(match);
            const prevText = template.slice(0, index);
            if (prevText)
                result.push({ type: 'text', data: { text: prevText } });
            template = template.slice(index + match.length);
            const [typeWithPrefix, ...attrs] = match.slice(1, -1).split(',');
            const type = typeWithPrefix.replace('CQ:', '');
            const data = Object.fromEntries(attrs.map(attrStr => {
                const [key, ...valueArr] = attrStr.split('=');
                return [key, (0, zhin_1.escape)(valueArr.join('='))];
            }));
            result.push({
                type,
                data,
            });
        }
        if (template.length) {
            result.push({
                type: 'text',
                data: {
                    text: template,
                },
            });
        }
        return result;
    }
    Message.parseSegmentsFromCqCode = parseSegmentsFromCqCode;
    function formatSegments(message) {
        const result = [];
        if (!Array.isArray(message))
            message = [message];
        for (const item of message) {
            if (typeof item === 'string')
                result.push(...(0, zhin_1.parseFromTemplate)(item));
            else
                result.push({
                    type: item.type,
                    data: (0, zhin_1.valueMap)(item.data, zhin_1.unescape),
                });
        }
        return result;
    }
    Message.formatSegments = formatSegments;
    function formatToString(message) {
        if (typeof message === 'string')
            return formatToString(parseSegmentsFromCqCode(message));
        let result = '';
        for (let item of message) {
            if (typeof item == 'string')
                item = { type: 'text', data: { text: item } };
            const { type, data } = item;
            if (type === 'text')
                result += data.text || '';
            else
                result += `<${type} ${Object.entries(data)
                    .map(([key, value]) => {
                    if (key === 'qq' && type === 'at')
                        key = 'user_id';
                    return `${key}='${(0, zhin_1.escape)(JSON.stringify(value))}'`;
                })
                    .join(' ')}>`;
        }
        return result;
    }
    Message.formatToString = formatToString;
})(Message || (exports.Message = Message = {}));
